//                                                                                                                            
// ********************************************************************                                                       
// *                                                                  *                                                       
// * Physical Target and Detector Definitions                         *                                                       
// *                                                                  *                                                       
// ********************************************************************                                                       
//                                                                                                                            

#include "myDetectorConstruction.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4Sphere.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4MultiFunctionalDetector.hh"
#include "G4VPrimitiveScorer.hh"
#include "G4PSSphereSurfaceCurrent.hh"
#include "G4SDParticleFilter.hh"
#include "G4SDManager.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4ios.hh"
#include "G4NistManager.hh"
#include <stdio.h>
//#include "triPolRingSD.hh"
#include "G4UniformMagField.hh"
#include "G4FieldManager.hh"
#include "G4TransportationManager.hh"
#include "G4SubtractionSolid.hh"

#include "G4SystemOfUnits.hh"
// ********************************************************************                                                       

myDetectorConstruction::myDetectorConstruction() { }

// ********************************************************************                                                       

myDetectorConstruction::~myDetectorConstruction() { }

// ********************************************************************                 
G4VPhysicalVolume* myDetectorConstruction::Construct()
{

  //--------- Material definition ---------                                                                                     

  G4double density;

  G4NistManager* man = G4NistManager::Instance();

  G4double atomicNumber=1.;
  G4double massOfMole=1.008*g/mole;
  density=1.e-25*g/cm3;
  G4double temperature=2.73*kelvin;
  
  G4double pressure=3.e-18*pascal; //SPACE                                                                                  
  //G4double pressure=1.e-3*pascal;
  G4Material *Vacuum =  new G4Material("Vacuum",atomicNumber,massOfMole,density,kStateGas,temperature,pressure);

  //G4Material* Beryllium = man->FindOrBuildMaterial("G4_Be");
  G4Material* Blood = man->FindOrBuildMaterial("G4_BLOOD_ICRP");  
  //G4Material* Carbon = man->FindOrBuildMaterial("G4_C");
  //G4Material* CsI = man->FindOrBuildMaterial("G4_CESIUM_IODIDE");

  // Print all the materials defined.                                                                             
            
  G4cout << G4endl << "The materials defined are : " << G4endl << G4endl;
  G4cout << *(G4Material::GetMaterialTable()) << G4endl;

  //--------- Definitions of Solids, Logical Volumes, Physical Volumes ---------                                                

  //------------------------------                                                                                            
  // World                                                                                                                    
  //------------------------------                                                                                            

  G4ThreeVector positionWorld  = G4ThreeVector(0,0,0);
  solidWorld = new G4Box("world",50*cm,50*cm,75*cm);
  logicWorld = new G4LogicalVolume(solidWorld,Vacuum,"World",0,0,0);
  physiWorld = new G4PVPlacement(   0, // rotate matrix
						       positionWorld,     // at (x,y,z)
						       logicWorld,        // its logical volume
						       "World",           // its name
						       0,                 // its mother  volume
						       false,             // no boolean operations
						       0);                // copy number
 
  G4ThreeVector positionTestBox  = G4ThreeVector(0*cm,0.0*cm,0.0*cm);
  G4Box *solidTestBox = new G4Box("TestBox",1*cm,1*cm,3*cm);
  G4LogicalVolume *logicTestBox = new G4LogicalVolume(solidTestBox,Blood,"TestBox",0,0,0);
  G4VPhysicalVolume *physiTestBox = new G4PVPlacement(   0, // rotate matrix
  							 positionTestBox,     // at (x,y,z)
  							 logicTestBox,        // its logical volume
  							 "TestBox",           // its name
  							 logicWorld,        // its mother  volume
  							 false,             // no boolean operations
  							 0);                // copy number


  //--------- Visualization attributes -------------------------------   
  G4VisAttributes* WorldVisAtt= new G4VisAttributes(G4Colour(0.0,0.0,0.0));
  logicWorld->SetVisAttributes(WorldVisAtt);
  // logicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4VisAttributes* redSolidVA = new G4VisAttributes(G4Colour(1.0,0.0,0.0));
  //redSolidVA->SetForceSolid(true);

  G4VisAttributes* greenSolidVA = new G4VisAttributes(G4Colour(0.0,1.0,0.0));
  greenSolidVA->SetForceSolid(true);

  G4VisAttributes* blueSolidVA = new G4VisAttributes(G4Colour(0.0,0.0,1.0));
  blueSolidVA->SetForceSolid(true);

  G4VisAttributes* whatSolidVA = new G4VisAttributes(G4Colour(0.5,0.0,1.0));
  whatSolidVA->SetForceSolid(true);


  logicTestBox->SetVisAttributes(redSolidVA);


  return physiWorld;
}








